# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import Agent, AgentCreate, AgentUpdate


class AgentsService(BaseService):

    @cast_models
    def create_agent(self, request_body: AgentCreate) -> Agent:
        """Create a new agent with the specified configuration. Supports both standard and platform agents.

        :param request_body: The request body.
        :type request_body: AgentCreate
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Agent
        """

        Validator(AgentCreate).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/agents", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, _, _ = self.send_request(serialized_request)
        return Agent._unmap(response)

    @cast_models
    def get_agent(self, agent_id: str) -> Agent:
        """Retrieve detailed information about a specific agent, including its configuration and settings

        :param agent_id: Unique identifier of the agent
        :type agent_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Agent
        """

        Validator(str).validate(agent_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/agents/{{agent_id}}", self.get_default_headers()
            )
            .add_path("agent_id", agent_id)
            .serialize()
            .set_method("GET")
        )

        response, _, _ = self.send_request(serialized_request)
        return Agent._unmap(response)

    @cast_models
    def update_agent(self, request_body: AgentUpdate, agent_id: str) -> None:
        """Update an existing agent's properties

        :param request_body: The request body.
        :type request_body: AgentUpdate
        :param agent_id: agent_id
        :type agent_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(AgentUpdate).validate(request_body)
        Validator(str).validate(agent_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/agents/{{agent_id}}", self.get_default_headers()
            )
            .add_path("agent_id", agent_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        self.send_request(serialized_request)

    @cast_models
    def delete_agent(self, agent_id: str) -> None:
        """Permanently delete an agent

        :param agent_id: agent_id
        :type agent_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(agent_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/agents/{{agent_id}}", self.get_default_headers()
            )
            .add_path("agent_id", agent_id)
            .serialize()
            .set_method("DELETE")
        )

        self.send_request(serialized_request)
